<?php

namespace Teknisa\MAN\Services;

use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ParaVendasImposto {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
		$this->connection = $entityManager->getConnection();
	}
	
	public function getTipoImposto($CDFILIAL, $NRORG, $CDIMPOSTO){
		$params = array( 
			':CDFILIAL'=> $CDFILIAL,
			':NRORG'   => $NRORG
		);
		
		if($CDIMPOSTO==99){
		 return $this->entityManager->getConnection()->fetchAll(self::TIPO_IMPOSTO_OUTROS, $params);
		}
		else
		{
		 return $this->entityManager->getConnection()->fetchAll(self::TIPO_IMPOSTO, $params);
	   }
	}
	
	public function deleteProdutosImposto($products) {
		try {
			foreach($products as $product) {
				$this->entityManager->getConnection()->executeQuery(Utilities\Query::DELETE_IMPOSTO_PRODUTO, $product);
			}
		} catch(\Exception $e) {
			var_dump($e);
		}
	}
	
	
	public function deleteProdutoImposto($product) {
		try {
			$params = array(
				"CDPRODUTO" => $product["CDPRODUTO"],
				"NRORG" 	=> $product["NRORG"],
				"CDIMPOSTO" => $product["CDIMPOSTO"],
				"CDFILIAL"	=> $product["CDFILIAL"] 
			);
			$this->entityManager->getConnection()->executeQuery(Utilities\Query::DELETE_IMPOSTO_PRODUTO, $params);
		} catch(\Exception $e) {
			var_dump($e->getMessage());
		}
	}
	
	public function saveProdutosImposto($products, $params) {
		try {	
			foreach($products as $product) {
				$row = array();
				foreach($params as $key => $value) {
					$row[$key] = $value;
				}
				$row['CDPRODUTO'] = $product['CDPRODUTO'];
				$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
								
		   }
		}catch(\Exception $e) {
			var_dump($e->getMessage());
		}
	}
	
	public function saveProdutosImpostoLista($products, $params) {
		try {	
			foreach($products as $product) {
				$row = array();
				foreach($params as $key => $value) {
					$row[$key] = $value;
				}
				$row['CDPRODUTO'] = $product;
				$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
			}
		} catch(\Exception $e) {
			var_dump($e->getMessage());
		}
	}

	public function saveProdutoImposto($product) {
		try {	
			$row = array();
			foreach($product as $key => $value) {
				$row[$key] = $value;
			}
			$this->entityManager->getConnection()->executeQuery(self::INSERE_IMPOSTO_PRODUTO, $row);
			
		} catch(\Exception $e) {
	     	var_dump($e->getMessage());

		}
	}
	
	public function generateAliquote ($row) {
        $row["P_CDREGRAFISC"]  = $this->buildInputParam("P_CDREGRAFISC", $row["CDFILIAL"]);
        
        $this->genAliProcedure($row);
    }
    
    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }
    
    public function genAliProcedure ($params) {
	    try{  
	        $storedProcedure = new StoredProcedure($this->connection, 'GERA_ALIQUOTA_VND');
	        $storedProcedure->addParam($params["P_CDREGRAFISC"]);
	        $storedProcedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    public function findProdutosNivel($row){
    	try{
    		$row['LIMIT_CDARVPROD'] = isset($row['LIMIT_CDARVPROD']) ? $row['LIMIT_CDARVPROD'] : $row['CDARVPROD'];
    		$params = array(
         	":CDFILIAL" 		=>	$row['CDFILIAL'],
         	":CDNVPRODUTO"		=>	$row['CDNVPRODUTO'],
         	":LIMIT_CDARVPROD"	=>	$row['LIMIT_CDARVPROD'],
         	":NRORG"			=>	$row['NRORG']
     		);
	
     		$response = $this->entityManager->getConnection()->fetchAll(self::FIND_PRODUTOS_FIELD, $params);
     		
    		return $response;
    		
    	} catch(\Exception $e){
    		throw $e;
    	}
    }
    
    public function validaObrigatorio($row){
		$array = array(
            "status" => true,
            "message" => ''
        );
      
        
        if($row['CDCFOPPFIS']=='5405'||$row['CDCFOPPFIS']=='5656'||$row['CDCFOPPFIS']=='5667'){
        	if($row['CDCSTICMS']!='60' && $row['CDCSTICMS']!='500'){
        	$array['status'] = false;
        	$array['message'] = "O código CFOP selecionado só pode ser usado com o CST/CSOSN 60 OU 500";
        	
        	return $array;
        	}
        }
        
        
        if($row['CDCSTPISCOF']=='49'||$row['CDCSTPISCOF']=='03'){
        	$result = $this->verificarCaixaNFC($row);
        	if(!$result){
        		$array['status'] = false;
        		$array['message'] = "O código CST (PIS/COFINS) 49 ou 03 so pode ser usado em filiais que possuem caixa tipo SAT.";
        	
        		return $array;
        	}
        }else{
		     return $array;
		 }
        
	}
    
    public function verificarCaixaNFC($row){

    	try{
 
    		$params = array(
         	":CDFILIAL" 		=>	$row["CDFILIAL"]
     		);
     		
     		
     		$response = $this->entityManager->getConnection()->fetchAll(self::FIND_CODIGO, $params);
     		
    		return $response;
    		
    	} catch(\Exception $e){
    		throw $e;
    	}
    }
    
    public function getParamentrosFind ($conditions){

    	$params = [];
        foreach ($conditions as $value) {
            $params[$value['columnName']] = $value['value'];
        }
    	$adqModProdFili = $this->entityManager->getConnection()->fetchAssoc(self::adqModProdFili, $params);
    	$adqGrupoPDV = $this->entityManager->getConnection()->fetchAssoc(self::adqGrupoPDV, $params);
    	$cdSegMer = $adqGrupoPDV['CDSEGMER'];
    	$dataSourceName = '';
    	if($adqModProdFili['IDUTMODPRODFIL'] == 'S'){
    		$dataSourceName = 'filterModProd';
    	}else{
    		if(!empty($adqGrupoPDV['CDGRPRODPDV'])){
    			if(!empty($adqGrupoPDV['CDSEGMER'])){
    				// Utiliza família / segmento
    				$dataSourceName = 'filterImpProdSegFam';
    			}else{
    				// Utiliza família
    				$dataSourceName = 'filterImpProdFamilia';
    			}
    		}else{
    			if(!empty($adqGrupoPDV['CDSEGMER'])){
    				// Utiliza segmento
    				$dataSourceName = 'filterImpProdSegmento';
    			}else{
    				// Não utiliza nada
    				$dataSourceName = 'filterImpProdTodos';
    			}
    		}
    	}
    	
    	return array('CDSEGMER'=>$cdSegMer,'DATA_SOURCE'=>$dataSourceName);
    }
    
    const adqGrupoPDV = "
    	SELECT PV.CDGRPRODPDV,
		  FI.CDSEGMER
		FROM FILIAL FI,
		  PARAVEND PV
		WHERE (FI.CDFILIAL = PV.CDFILIAL (+))
		AND (FI.CDFILIAL   = :CDFILIAL) AND (FI.NRORG   = :NRORG)
    ";
    
    const adqModProdFili = "
    	SELECT P.CDFILIAL,
		  P.IDUTMODPRODFIL
		FROM PARAMFILIAL P
		WHERE (P.CDFILIAL = :CDFILIAL)
		AND (P.NRORG      = :NRORG)
    ";
    const FIND_CODIGO = "
	SELECT
	    *
	FROM
	    CAIXA
	WHERE
	    IDTPEMISSAOFOS IN ('FNC','SAT')
	    AND CDFILIAL = :CDFILIAL
    ";
    
    const INSERE_IMPOSTO_PRODUTO = "    
    INSERT INTO ALIQIMPFIS 
      (VRPEALIMPFIS, VRPERCREDUCAO, IDMODALBASECALC, IDINCIDEPISCOF, VRALIQIBPT, 
       VRALIQIBPTES, CDCFOPPFIS, CDCSTICMS, CDCSTPISCOF, VRALIQPIS, VRALIQCOFINS,
       CDCEST, CDIMPOSTO, CDFILIAL, NRORG, CDPRODUTO, IDTPIMPOSFIS, VRALIQFCP, VRPEALIMPFISEF, VRPERCREDUCAOEF, CDCBENEF, VRALIQICMSDES)
    VALUES 
      (:VRPEALIMPFIS, :VRPERCREDUCAO, :IDMODALBASECALC, :IDINCIDEPISCOF, :VRALIQIBPT, 
       :VRALIQIBPTES, :CDCFOPPFIS, :CDCSTICMS, :CDCSTPISCOF, :VRALIQPIS, :VRALIQCOFINS,
       :CDCEST, :CDIMPOSTO, :CDFILIAL, :NRORG, :CDPRODUTO, :IDTPIMPOSFIS, :VRALIQFCP, :VRPEALIMPFISEF, :VRPERCREDUCAOEF, :CDCBENEF, :VRALIQICMSDES)";  
    
    const FIND_PRODUTOS_FIELD = "
    	SELECT P.NRORG, P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.CDARVPROD || ' - ' || P.NMPRODUTO AS PRODUTOSELECAO 
		    FROM PRODUTO P, PRODFILI F, 
		            (SELECT AL.CDFILIAL, AL.CDPRODUTO, AL.VRALIQFCP, AL.VRPERCREDUCAOEF, AL.VRALIQICMSDES, AL.CDCBENEF, AL.VRPEALIMPFISEF, AL.CDIMPOSTO, AL.VRPEALIMPFIS, AL.IDTPIMPOSFIS, IM.SGIMPOSTO, IM.NMIMPOSTO, 
		            AL.IDINCIDEPISCOF, AL.VRALIQIBPT, AL.VRALIQIBPTES, AL.CDCSTICMS, AL.CDCSTPISCOF, AL.CDCFOPPFIS, AL.VRALIQPIS, 
		            AL.VRALIQCOFINS, AL.VRPERCREDUCAO, AL.IDMODALBASECALC, AL.CDCEST, AL.NRORG 
		                FROM ALIQIMPFIS AL, IMPOSTO IM 
		                WHERE AL.CDIMPOSTO = IM.CDIMPOSTO 
		                AND AL.CDFILIAL = :CDFILIAL 
		                AND AL.NRORG = IM.NRORG 
		                AND AL.NRORG = :NRORG ) A
		        WHERE (NVL(P.IDPRODATIVPR,'S') = 'S') 
		        AND P.CDARVPROD >= :LIMIT_CDARVPROD 
		        AND (P.CDNVPRODUTO = :CDNVPRODUTO)
		        AND (P.CDPRODUTO = F.CDPRODUTO(+)) 
		        AND (:CDFILIAL = F.CDFILIAL (+)) 
		        AND (P.CDPRODUTO = A.CDPRODUTO(+)) 
		        AND (P.NRORG = A.NRORG(+)) 
		        AND (P.NRORG = F.NRORG(+)) 
		        AND P.NRORG = :NRORG 
		ORDER BY NLSSORT(P.CDARVPROD, 'NLS_SORT=POLISH')
    ";
    const TIPO_IMPOSTO= "
         SELECT P.CDSITUTRIB, P.DSSITUTRIB 
          FROM FILIAL F 
          JOIN PARAMEMPRESA PF 
            ON F.CDEMPRESA = PF.CDEMPRESA 
           AND F.NRORG = PF.NRORG 
          JOIN PARAMCST P 
            ON P.NRORG = F.NRORG 
         WHERE (P.CDIMPOSTO = '00') 
           AND (P.IDENTRSAIDOP = 'S') 
           AND PF.CDSITUCRT = '3'
           AND F.CDFILIAL = :CDFILIAL
           AND F.NRORG = :NRORG
           AND  P.CDSITUTRIB IN ('00','20','40','41','60','90')
    ";
    const TIPO_IMPOSTO_OUTROS= "
         SELECT P.CDSITUTRIB, P.DSSITUTRIB 
          FROM FILIAL F 
          JOIN PARAMEMPRESA PF 
            ON F.CDEMPRESA = PF.CDEMPRESA 
           AND F.NRORG = PF.NRORG 
          JOIN PARAMCST P 
            ON P.NRORG = F.NRORG 
         WHERE (P.CDIMPOSTO = '00') 
           AND (P.IDENTRSAIDOP = 'S') 
           AND PF.CDSITUCRT = '3'
           AND F.CDFILIAL = :CDFILIAL
           AND F.NRORG = :NRORG
           AND  P.CDSITUTRIB IN ('60')
    ";
}